#pragma once

#include <cstddef>

#include <VirtualRobot/CollisionDetection/CollisionModel.h>

#include "Node.h"
#include "Planner2D.h"

namespace armarx::navigation::algorithm::astar
{

    /**
 * The A* planner
 */
    class AStarPlanner : public Planner2D
    {
    public:
        EIGEN_MAKE_ALIGNED_OPERATOR_NEW

        AStarPlanner(VirtualRobot::RobotPtr robot,
                     VirtualRobot::SceneObjectSetPtr obstacles = {},
                     float cellSize = 100.f);

        std::vector<Eigen::Vector2f> plan(const Eigen::Vector2f& start, const Eigen::Vector2f& goal) override;

        void setRobotCollisionModel(const VirtualRobot::CollisionModelPtr& collisionModel);

    private:
        float heuristic(const NodePtr& n1, const NodePtr& n2);
        void createUniformGrid();
        bool fulfillsConstraints(const NodePtr& n);
        NodePtr closestNode(const Eigen::Vector2f& v);

    private:
        /// How big each cell is in the uniform grid.
        float cellSize;

        std::vector<std::vector<NodePtr>> grid;

        size_t cols = 0;
        size_t rows = 0;
    };
} // namespace armarx::navigation::algorithm::astar
